import { assignDeepOmitBlank, getChartCategory, getEntityHierarchy, getEntityProperty, getPropertyCaption, getPropertyHierarchy, getPropertyMeasure, getPropertyName } from '@metad/ocap-core';
import { measuresToSeriesComponents, serializeSeriesComponent } from './cartesian';
import { formatMeasureNumber } from './common';
export function cartesian3d(data, chartAnnotation, entityType, settings, options, type) {
    const context = {
        data,
        chartAnnotation,
        entityType,
        settings,
        options
    };
    const cartesianCoordinate = cartesianCoordinate3d(data.data, chartAnnotation, entityType, settings, options);
    const echartsOptions = {
        dataset: [],
        grid3D: [],
        xAxis3D: [],
        yAxis3D: [],
        zAxis3D: [],
        series: [],
        visualMap: [],
        tooltip: []
    };
    [cartesianCoordinate].forEach((cartesianCoordinate) => {
        echartsOptions.grid3D.push(cartesianCoordinate.grid3D);
        echartsOptions.xAxis3D.push(cartesianCoordinate.xAxis3D);
        echartsOptions.yAxis3D.push(cartesianCoordinate.yAxis3D);
        echartsOptions.zAxis3D.push(cartesianCoordinate.zAxis3D);
        echartsOptions.visualMap.push(...cartesianCoordinate.visualMap);
        echartsOptions.tooltip.push(...cartesianCoordinate.tooltip);
        cartesianCoordinate.datasets.forEach(({ dataset, series }) => {
            echartsOptions.dataset.push(dataset);
            series.forEach((series) => {
                echartsOptions.series.push({
                    ...series,
                    datasetIndex: echartsOptions.dataset.length - 1,
                    type: series.type ?? type
                });
            });
        });
    });
    return {
        ...context,
        echartsOptions
    };
}
// 单个笛卡尔坐标系
export function cartesianCoordinate3d(data, chartAnnotation, entityType, settings, options) {
    const category = getChartCategory(chartAnnotation);
    const category2 = chartAnnotation.dimensions[1];
    const xHierarchy = getEntityHierarchy(entityType, category);
    const yHierarchy = getEntityHierarchy(entityType, category2);
    const mainMeasure = chartAnnotation.measures[0];
    // const mainMeasureProperty = getEntityProperty(entityType, mainMeasure)
    const datasets = [
        {
            dataset: {
                source: data,
                dimensions: [
                    ...chartAnnotation.dimensions.map(getPropertyName),
                    ...chartAnnotation.measures.map(getPropertyMeasure)
                ]
            },
            seriesComponents: measuresToSeriesComponents(chartAnnotation.measures, data, entityType, settings),
        }
    ];
    const tooltips = [
        {
            value: getPropertyCaption(xHierarchy),
            label: xHierarchy.caption || xHierarchy.name,
            valueFormatter: null
        },
        {
            value: getPropertyCaption(yHierarchy),
            label: yHierarchy.caption || yHierarchy.name,
            valueFormatter: null
        },
        ...chartAnnotation.measures.map((measure) => {
            const measureProperty = getEntityProperty(entityType, measure);
            return {
                value: measure.measure,
                label: getEntityProperty(entityType, measure).caption,
                valueFormatter: (value) => {
                    return formatMeasureNumber({ measure: measure, property: measureProperty }, value, settings.locale, measure.formatting?.shortNumber);
                }
            };
        }),
    ];
    const grid3dOptions = {
        grid3D: assignDeepOmitBlank({
            top: 0,
            right: 0,
            bottom: 0,
            left: 0,
            containLabel: true
        }, options?.grid3D),
        xAxis3D: assignDeepOmitBlank({
            type: 'category',
            name: xHierarchy.caption,
            axisLabel: {
                formatter: (key, tick, ...params) => {
                    const caption = data.find((item) => item[xHierarchy.name] === key)?.[xHierarchy.memberCaption];
                    return caption || key;
                }
            }
        }, options?.xAxis3D),
        yAxis3D: assignDeepOmitBlank({
            type: 'category',
            name: yHierarchy.caption,
            axisLabel: {
                formatter: (key, tick, ...params) => {
                    const caption = data.find((item) => item[yHierarchy.name] === key)?.[yHierarchy.memberCaption];
                    return caption || key;
                }
            },
        }, options?.yAxis3D),
        zAxis3D: assignDeepOmitBlank(assignDeepOmitBlank({
            type: 'value'
        }, options?.zAxis3D), mainMeasure.chartOptions?.axis),
        visualMap: [],
        datasets: [],
        tooltip: [
            assignDeepOmitBlank({
                formatter: (params) => {
                    return params.marker + tooltips.map(({ value, label, valueFormatter }) => label + ': ' + (valueFormatter ? valueFormatter(params.data[value]) : params.data[value])).join('<br>');
                },
            }, options?.tooltip)
        ]
    };
    datasets.forEach(({ dataset, seriesComponents }) => {
        grid3dOptions.datasets.push({
            dataset,
            series: seriesComponents.map((seriesComponent) => {
                const { series, visualMaps } = serializeSeriesComponent(dataset, seriesComponent, entityType, 'z', settings, options);
                grid3dOptions.visualMap.push(...visualMaps);
                series.encode = {
                    x: getPropertyHierarchy(category),
                    y: getPropertyHierarchy(category2),
                    z: seriesComponent.measure,
                };
                return assignDeepOmitBlank(series, options?.seriesStyle);
            })
        });
    });
    return grid3dOptions;
}
//# sourceMappingURL=cartesian3d.js.map